# In order to source this TCL script in System-Console, --project_dir of System-Console 
# must be specified. This is to ensure each jtag nodes in FPGA design are detectable via 
# its node name.

# setting parameter for system infomation ######################################
# this section must be modified to represent the targeted memory map of the design
set sysid_base 0x10000
set pio_led_base 0x10040
set pio_dipsw_base 0x10080
set pio_button_base 0x100c0
set uart_base 0x10100
set intr_capture_base 0x30000
set juart_base 0x20000
set onchip_mem_base 0x0
set hps_led_base 0xff709000
set tse0_tx_descp_base 0x420
set tse0_rx_descp_base 0x480
set tse1_tx_descp_base 0x1420
set tse1_rx_descp_base 0x1480
set tse0_tx_csr_base   0x400
set tse0_rx_csr_base   0x460
set tse1_tx_csr_base   0x1400
set tse1_rx_csr_base   0x1460
set tse0_ctrl_base     0x0
set tse1_ctrl_base     0x1000


set uart_int_bit 3
set dipsw_int_bit 4
set txdma_int_bit 0
set button_int_bit 5
set rxdma_int_bit 1
set juart_int_bit 2

################################################################################

# set $mj as handler of dedicated master
#set mj [get_service_paths master]

# if system has multiple masters, dedicated master to be in variable
# so to know that return master service path with phy_0 is an non-secure jtag master,
# phy_1 is an secure jtag master in design, and phy_2 is JTAG master attached to SGDMA

set all_sp [get_service_paths master]
#puts "all ser.path: $all_sp"
puts "llength of all_sp: [ llength $all_sp ]"
for {set i 0} {$i<[llength $all_sp]} {incr i} {
  set sp($i) [lindex [get_service_paths master] $i]
  #puts $sp($i)

  if {[ regexp {/5CS.+/fpga_only_master} "$sp($i)" ]} {
    #set result [ regexp {/phy_1/} "$sp($i)" ]
    #puts "matched /phy_0/: $result"
    set mj_nsec "$sp($i)"
    puts "non-secure jtag master: $mj_nsec"
    open_service master $mj_nsec
    }

  if {[ regexp {/5CS.+/hps_only_master} "$sp($i)" ]} {
    #set result [ regexp {/phy_0/} "$sp($i)" ]
    #puts "matched /phy_1/: $result"
    set mj_secu "$sp($i)"
    puts "secure jtag master    : $mj_secu"
    open_service master $mj_secu
    }
    
  if {[ regexp {/5CS.+/dma_descp_master} "$sp($i)" ]} {
    #set result [ regexp {/phy_0/} "$sp($i)" ]
    #puts "matched /phy_1/: $result"
    set mj_dmad "$sp($i)"
    puts "dma descriptor jtag master: $mj_dmad"
    open_service master $mj_dmad
    }
  }
 

set ser_path [get_service_paths monitor]
set mon_path [claim_service monitor $ser_path "my_lib" ""]
monitor_set_interval $mon_path 5000
monitor_set_callback $mon_path [list monitor_callback_bg $mon_path]
monitor_add_range $mon_path $mj_nsec $intr_capture_base 1

# disable the monitoring activity by default when script is sourced
#monitor_set_enabled $mon_path 0

proc monitor_callback_bg {mon_path} {
  global mj_nsec
  global intr_capture_base
  global uart_int_bit
  global txdma_int_bit
  global rxdma_int_bit
  global juart_int_bit

  set int_data [monitor_read_data $mon_path $mj_nsec $intr_capture_base 1]
  set interv [monitor_get_read_interval $mon_path $mj_nsec $intr_capture_base 1]
  puts "\nInterrupt capture value $int_data at interval of $interv ms"


  set uart_irq_locate [ expr { $int_data >> $uart_int_bit } ]
  #puts "located uart = $uart_irq_locate"
  set is_uart_irq [ expr {$uart_irq_locate & 1} ]
  puts "uart irq value: $is_uart_irq"
  if { $is_uart_irq != 0 } {
    puts "     UART_0 got interrupt!"
  }

  set dipsw_irq_locate [ expr { $int_data >> $txdma_int_bit } ]
  #puts "located dipsw = $dipsw_irq_locate"
  set is_dipsw_irq [ expr {$dipsw_irq_locate & 1} ]
  puts "dipswitch irq value: $is_dipsw_irq"
  if { $is_dipsw_irq != 0 } {
    puts "     TX DMA got interrupt!"
  }

  set button_irq_locate [ expr { $int_data >> $rxdma_int_bit } ]
  #puts "located button = $button_irq_locate"
  set is_button_irq [ expr {$button_irq_locate & 1} ]
  puts "button irq value: $is_button_irq"
  if { $is_button_irq != 0 } {
    puts "     RX DMA got interrupt!"
  }

  set juart_irq_locate [ expr { $int_data >> $juart_int_bit } ]
  set is_juart_irq [ expr {$juart_irq_locate & 1} ]
  puts "juart irq value: $is_juart_irq"
  if { $is_juart_irq != 0 } {
    puts "     JTAG UART got interrupt!"
  }

  puts "wait for 5 seconds to next read..."
}

proc irq_monitor_on {} {
  global mon_path
  monitor_set_enabled $mon_path 1
}

proc irq_monitor_off {} {
  global mon_path
  monitor_set_enabled $mon_path 0
}

proc nsec_wr {addr char {readback 0}} {
  global mj_nsec
  master_write_32 $mj_nsec $addr $char
  if {$readback == 1} {
    set readdata [master_read_32 $mj_nsec $addr 1]
    puts "data read back from address: 0x[ format %x [ expr $addr ] ] = $readdata"
  } else {
    puts "written to address: 0x[ format %x [ expr $addr ] ]"
  }
}

proc nsec_rd {addr {cnt 1}} {
  global mj_nsec
  set rdata [master_read_32 $mj_nsec $addr $cnt]
  puts "readdata: $rdata, from addr: 0x[ format %x $addr]"
  return $rdata
}

proc secu_wr {addr char {readback 0}} {
  global mj_secu
  master_write_32 $mj_secu $addr $char
  if {$readback == 1} {
    set readdata [master_read_32 $mj_secu $addr 1]
    puts "data read back from address: 0x[ format %x [ expr $addr ] ] = $readdata"
  } else {
    puts "written to address: 0x[ format %x [ expr $addr ] ]"
  }
}

proc secu_rd {addr {cnt 1}} {
  global mj_secu
  set rdata [master_read_32 $mj_secu $addr $cnt]
  puts "readdata: $rdata, from addr: 0x[ format %x $addr]"
  return $rdata
}

proc dmad_wr {addr char {readback 0}} {
  global mj_dmad
  master_write_32 $mj_dmad $addr $char
  if {$readback == 1} {
    set readdata [master_read_32 $mj_dmad $addr 1]
    puts "data read back from address: 0x[ format %x [ expr $addr ] ] = $readdata"
  } else {
    puts "written to address: 0x[ format %x [ expr $addr ] ]"
  }
}

proc dmad_rd {addr {cnt 1}} {
  global mj_dmad
  set rdata [master_read_32 $mj_dmad $addr $cnt]
  puts "readdata: $rdata, from addr: 0x[ format %x $addr]"
  return $rdata
}

proc int2bits {i} {
     #returns a bitslist, e.g. int2bits 10 => {1 0 1 0} 
     set res ""
     while {$i>0} {
         set res [expr {$i%2}]$res
         set i [expr {$i/2}]
     }
     if {$res==""} {set res 0}
     split $res ""
 }

# generate random integer number in the range [min,max]
proc randrange { min max } {
    set maxfactor [expr [expr $max + 1] - $min]
    set value [ expr int([expr rand() * $maxfactor])]
    set value [ expr $value + $min ]
   # set value [expr int([expr rand() * 100])]
   # set value [expr [expr $value % $maxfactor] + $min]
return $value
}

# procedure that read FPGA system ID
proc sysid_read {} {
  global mj_nsec
  global sysid_base
  set rdata [nsec_rd $sysid_base]
}

# procedure that read push button value
proc button_data_read {} {
  global mj_nsec
  global pio_button_base
  nsec_rd 0x[format %8.8x [ expr $pio_button_base+0xc ]]
}

# procedure that read push button value and write 1 to clear
proc button_data_read_clear {} {
  global mj_nsec
  global pio_button_base
  set rdata [ nsec_rd 0x[format %8.8x [ expr $pio_button_base+0xc ]]]
  puts "Button value: 0x$rdata\n"
  nsec_wr [ expr $pio_button_base+0xc ] $rdata
}

# procedure that enable interrupt of button peripheral
# "bit_enable" argument represent which bit of push button to generate IRQ
proc button_intr_enable {bit_enable} {
  global mj_nsec
  global pio_button_base
  set inhex [ expr 0x1<<$bit_enable ]
  #set inhex [format %x $bit_enable]
  set rintr [ nsec_rd [ expr $pio_button_base+8 ] ]
  puts "read existing interrupt reg: $rintr"
  set wintr [ expr $rintr | $inhex ]
  puts "irq enable value to be written: 0x[format %8.8x $wintr]"
  nsec_wr [ expr $pio_button_base+8 ] $wintr
}

# procedure that disable interrupt of button peripheral
proc button_intr_disable {} {
  global mj_nsec
  global pio_button_base
  nsec_wr [ expr $pio_button_base+8 ] 0x0
}

# procedure that light on LED base on 4-bit hexadecimal value written
proc led_on {pattern} {
  global mj_nsec
  global pio_led_base
  nsec_wr [ expr $pio_led_base ] $pattern
}

# procedure that light off the 4 LEDs. Value '1' at PIO will turn off the lighting
proc led_off {} {
  global mj_nsec
  global pio_led_base
  nsec_wr [ expr $pio_led_base ] 0xf
}

# procedure set FPGA LED to run in single light-on pattern, with an interval set by user
proc led_run {interval occurance} {
  global mj_nsec
  global pio_led_base
  #regrd 0x0
  for {set y 0} {$y<$occurance} {incr y} {
    set value 1    
    for {set x 0} {$x<4} {incr x} {
      set dvalue [expr ~$value]
      nsec_wr $pio_led_base $dvalue
      puts " value of LED PIO register~: 0b [int2bits $value]"
      set value [expr $value <<1 ]
      after $interval
    }
  }
}

# procedure to read LED PIO value
proc led_read {} {
  global mj_nsec
  global pio_led_base
  nsec_rd 0x[format %8.8x [ expr $pio_led_base ]]
}

# procedure that read dip switch value
proc dipsw_data_read {} {
  global mj_nsec
  global pio_dipsw_base
  nsec_rd 0x[format %8.8x [ expr $pio_dipsw_base+0x0 ]]
}

# procedure that read dip switch value
proc dipsw_data_read_toggle {} {
  global mj_nsec
  global pio_dipsw_base
  set data [nsec_rd 0x[format %8.8x [ expr $pio_dipsw_base+0x0 ]]]
  puts "DIP Switch data         : $data"
  set toggle [nsec_rd 0x[format %8.8x [ expr $pio_dipsw_base+0xc ]]]
  puts "DIP Switch toggled value: $toggle\n"
}

# procedure that read dip switch value
proc dipsw_data_read_toggle_clear {} {
  global mj_nsec
  global pio_dipsw_base
  set rdata [ nsec_rd 0x[format %8.8x [ expr $pio_dipsw_base+0x0 ]]]
  puts "DIP Switch data         : $rdata\n"
  set rtoggle [ nsec_rd 0x[format %8.8x [ expr $pio_dipsw_base+0xc ]]]
  puts "DIP Switch toggled value: $rtoggle\n"
  nsec_wr [ expr $pio_dipsw_base+0xc ] $rtoggle
}

# procedure that enable interrupt of dip switch peripheral
# "bit_enable" argument represent which bit of dip switch to be enable for IRQ generation
proc dipsw_intr_enable {bit_enable} {
  global mj_nsec
  global pio_dipsw_base
  set inhex [ expr 0x1<<$bit_enable ]
#  set inhex [format %x $bit_enable]
  set rintr [ nsec_rd [ expr $pio_dipsw_base+8 ] ]
  puts "read existing interrupt reg: $rintr"
  set wintr [ expr $rintr | $inhex ]
  puts "interrupt value to be written: 0x[format %8.8x $wintr]"
  nsec_wr [ expr $pio_dipsw_base+8 ] $wintr
}

# procedure that disable interrupt of dip switch peripheral
proc dipsw_intr_disable {} {
  global mj_nsec
  global pio_dipsw_base
  nsec_wr [ expr $pio_dipsw_base+8 ] 0x0
}

# procedure that read from jtag UART, one word at a time,
# "occurance" argument for multiple read
proc juart_data_read {{occurance 1}} {
  global mj_nsec
  global juart_base
  for {set oc 0} {$oc<$occurance} {incr oc} {
   nsec_rd $juart_base
  }
}

# procedure that write to jtag uart peripheral, one word at a time
proc juart_data_write {data} {
  global mj_nsec
  global juart_base
  nsec_wr 0x[ format %x [ expr $juart_base ] ] $data
}

# procedure that write random data to jtag UART 
proc juart_data_write_rand {} {
  global mj_nsec
  global juart_base
  set data [ randrange 32 127 ]
  nsec_wr [ expr $juart_base ] $data
  puts "written data: $data"
#[ expr {int(rand()*127)} ]
}

# procedure that write random data to jtag UART, 
# optional "occurance" argument for multiple write with random data each time
proc juart_data_write_rand_repeat {{occurance 1}} {
  global mj_nsec
  global juart_base
  for {set oc 0} {$oc<$occurance} {incr oc} {
    set data [ randrange 32 127 ]
#    set data [ expr {int(rand()*255)} ]
    nsec_wr [ expr $juart_base ] $data
#    set fdata [ format %4x $data ]
#    puts "written data: $fdata"
    puts "written data: $data"
  }
}

# procedure to write random data to random address of onchip memory and read back for comparison
proc rand_access {jtag_master_node base max_range_in_word {count 1}} {
  # puts "first text passed in: $jtag_master_node\n"
  global $jtag_master_node
  set match_count 0
  set miss_count 0
  for {set oc 0} {$oc<$count} {incr oc} {
    set dataarray($oc) [ expr {int(rand()*4294967296)} ]
    set addrarray($oc) [ expr [ expr {int(rand()*$max_range_in_word)} ] *4 ]
    if {$jtag_master_node == "mj_nsec"} {
      nsec_wr [ expr $base+$addrarray($oc) ] $dataarray($oc)
    }
    if {$jtag_master_node == "mj_secu"} {
      secu_wr [ expr $base+$addrarray($oc) ] $dataarray($oc)
    }
    if {$jtag_master_node == "mj_dmad"} {
      dmad_wr [ expr $base+$addrarray($oc) ] $dataarray($oc)
    }
  }
  puts "Done writting [expr $oc] set of data into location $base - 0x[format %8.8x [ expr $max_range_in_word*4 + $base ]]\n"
  puts "Start reading back for comparison"
  for {set oc 0} {$oc<$count} {incr oc} {
    if {$jtag_master_node == "mj_nsec"} {
      set data [ nsec_rd 0x[ format %x [ expr $base+$addrarray($oc) ] ] ]
    }
    if {$jtag_master_node == "mj_secu"} {
      set data [ secu_rd 0x[ format %x [ expr $base+$addrarray($oc) ] ] ]
    }
    if {$jtag_master_node == "mj_dmad"} {
      set data [ dmad_rd 0x[ format %x [ expr $base+$addrarray($oc) ] ] ]
    }
    if { $data != "0x[format %8.8x $dataarray($oc)]" } {
      puts "Data mismatch!\nwritten:0x[format %8.8x $dataarray($oc)], read:$data, at address:0x[format %8.8x [expr $addrarray($oc) + $base]]\n"
      set match_count [ expr $match_count+1 ]
    } else {
      puts "Data($oc) matched: $data\n"
      set miss_count [ expr $miss_count+1 ]
    }
  }
  puts "Total word transferred: $count"
  puts "Data missmatched: $match_count"
  puts "Data matched    : $miss_count" 
}

# procedure to write random data to incremental address of onchip memory and read back for comparison
proc incr_access {jtag_master_node base {count 1} {data "randomlist"}} {
  global $jtag_master_node
  puts "1st element data: [lindex $data 0]"
  puts "2st element data: [lindex $data 1]"
  puts "3st element data: [lindex $data 2]"
  puts "size of data passed: [llength $data]"
  set match_count 0
  set miss_count 0
  for {set oc 0} {$oc<$count} {incr oc} {
    if {$data == "randomlist"} {
      set dataarray($oc) [ expr {int(rand()*4294967296)} ]
    } elseif {[llength $data] > $oc} {
      set dataarray($oc) [lindex $data $oc]
    } else {
      set dataarray($oc) [lindex $data [expr [llength $data] -1]]
    }
    set addrarray($oc) [ expr $base + [ expr $oc*4 ]]
    if {$jtag_master_node == "mj_nsec"} {
      nsec_wr [ expr $addrarray($oc) ] $dataarray($oc)
    }
    if {$jtag_master_node == "mj_secu"} {
      secu_wr [ expr $addrarray($oc) ] $dataarray($oc)
    }
    if {$jtag_master_node == "mj_dmad"} {
      dmad_wr [ expr $addrarray($oc) ] $dataarray($oc)
    }
  }
  puts "Done writting [expr $oc] set of data incremental into location $base\n"
  puts "Start reading back for comparison"
  for {set oc 0} {$oc<$count} {incr oc} {
    if {$jtag_master_node == "mj_nsec"} {
      set data [ nsec_rd 0x[ format %x [ expr $addrarray($oc) ] ] ]
    }
    if {$jtag_master_node == "mj_secu"} {
      set data [ secu_rd 0x[ format %x [ expr $addrarray($oc) ] ] ]
    }
    if {$jtag_master_node == "mj_dmad"} {
      set data [ dmad_rd 0x[ format %x [ expr $addrarray($oc) ] ] ]
    }    
    if { $data != "0x[format %8.8x $dataarray($oc)]" } {
      puts "Data mismatch!\nwritten:0x[format %8.8x $dataarray($oc)], read:$data, at address:0x[format %8.8x $addrarray($oc)]\n"
      set match_count [ expr $match_count+1 ]
    } else {
      puts "Data($oc) matched: $data\n"
      set miss_count [ expr $miss_count+1 ]
    }
  }
  puts "Total word transferred: $count"
  puts "Data missmatched: $match_count"
  puts "Data matched    : $miss_count" 
}




# procedure to write random data to random address of onchip memory and read back for comparison
proc mem_rand_test {{count 1}} {
  global mj_nsec
  global onchip_mem_base
  set mm_count 0
  set mt_count 0
  for {set oc 0} {$oc<$count} {incr oc} {
    set dataarray($oc) [ expr {int(rand()*4294967296)} ]
    set addrarray($oc) [ expr [ expr {int(rand()*16380)} ] *4 ]
    nsec_wr [ expr $onchip_mem_base+$addrarray($oc) ] $dataarray($oc)
  }
  puts "Done writting [expr $oc] set of data into Onchip Memory\n"
  puts "Start reading back for comparison"
  for {set oc 0} {$oc<$count} {incr oc} {
    set data [ nsec_rd 0x[ format %x [ expr $onchip_mem_base+$addrarray($oc) ] ] ]
    if { $data != "0x[format %8.8x $dataarray($oc)]" } {
      puts "Data mismatch!\nwritten:0x[format %8.8x $dataarray($oc)], read:$data, at address:0x[format %8.8x $addrarray($oc)]\n"
      set mm_count [ expr $mm_count+1 ]
    } else {
      puts "Data($oc) matched: $data\n"
      set mt_count [ expr $mt_count+1 ]
    }
  }
  puts "Total word transferred: $count"
  puts "Data missmatched: $mm_count"
  puts "Data matched    : $mt_count" 
}

# procedure to write random data to incremental address of onchip memory and read back for comparison
proc mem_incr_test {startaddr {count 1}} {
  global mj_nsec
  global onchip_mem_base
  set mm_count 0
  set mt_count 0
  for {set oc 0} {$oc<$count} {incr oc} {
    set dataarray($oc) [ expr {int(rand()*4294967296)} ]
    set addrarray($oc) [ expr $startaddr + [ expr $oc*4 ]]
    nsec_wr [ expr $onchip_mem_base+$addrarray($oc) ] $dataarray($oc)
  }
  puts "Done writting [expr $oc] set of data into Onchip Memory\n"
  puts "Start reading back for comparison"
  for {set oc 0} {$oc<$count} {incr oc} {
    set data [ nsec_rd 0x[ format %x [ expr $onchip_mem_base+$addrarray($oc) ] ] ]
    if { $data != "0x[format %8.8x $dataarray($oc)]" } {
      puts "Data mismatch!\nwritten:0x[format %8.8x $dataarray($oc)], read:$data, at address:0x[format %8.8x $addrarray($oc)]\n"
      set mm_count [ expr $mm_count+1 ]
    } else {
      puts "Data($oc) matched: $data\n"
      set mt_count [ expr $mt_count+1 ]
    }
  }
  puts "Total word transferred: $count"
  puts "Data missmatched: $mm_count"
  puts "Data matched    : $mt_count" 
}

# procedure that read from UART recieve fifo, one word at a time,
# "occurance" argument for multiple read
proc uart_data_read {{occurance 1}} {
  global mj_nsec
  global uart_base
  for {set oc 0} {$oc<$occurance} {incr oc} {
    nsec_rd $uart_base
  }
}

# procedure that write to UART transmit fifo, one word at a time, 
proc uart_data_write {data} {
  global mj_nsec
  global uart_base
  nsec_wr [ expr $uart_base+4 ] $data
#  regwr 0x[ format %x [ expr $uart_base+4 ] ] $data
}

# procedure that write to UART transmit fifo, one word at a time, 
# "occurance" argument for multiple write with same data
proc uart_data_write_repeat {data {occurance 1}} {
  global mj_nsec
  global uart_base
  for {set oc 0} {$oc<$occurance} {incr oc} {
    nsec_wr [ expr $uart_base+4 ] $data
  }
}

# procedure that write random data to UART transmit fifo, one word at a time, 
proc uart_data_write_rand {} {
  global mj_nsec
  global uart_base
  nsec_wr [ expr $uart_base+4 ] [ expr {int(rand()*255)} ]
#  regwr 0x[ format %x [ expr $uart_base+4 ] ] [ expr {int(rand()*255)} ]
}

# procedure that write random data to UART transmit fifo, 
# "occurance" argument for multiple write with random data each time
proc uart_data_write_rand_repeat {{occurance 1}} {
  global mj_nsec
  global uart_base
  for {set oc 0} {$oc<$occurance} {incr oc} {
    set data [ expr {int(rand()*255)} ]
    nsec_wr [ expr $uart_base+4 ] $data
    # regwr 0x[ format %x [ expr $uart_base+4 ] ] $data
    set fdata [ format %4x $data ]
    puts "written data: $fdata"
  }
}

# procedure that read interrupt status of UART
proc uart_intr_status {} {
  global mj_nsec
  global uart_base
  nsec_rd 0x[ format %x [ expr $uart_base+8 ] ]
}

# procedure that enable interrupt generation of UART
# "intr_value" argument describes which interrupt bit to be set, it will be or with existing interrupt register value
proc uart_intr_set {intr_value} {
  global mj_nsec
  global uart_base
  set int_intr_value [ expr $intr_value ]
#  set inhex [format %x $bit_enable]
  set rintr [ nsec_rd 0x[ format %x [ expr $uart_base+12 ] ] ]
  puts "read existing interrupt reg: $rintr"
  set int_rintr [ expr $rintr]
  set wintr [ expr $int_rintr | $int_intr_value ]
  puts "interrupt value to be written: 0x[format %8.8x $wintr]"
  nsec_wr [ expr $uart_base+12 ] $wintr
}

# procedure that clear interrupt setting of UART
proc uart_intr_disable {} {
  global mj_nsec
  global uart_base
  nsec_wr [ expr $uart_base+12 ] 0x0
}

# procedure that light on HPS GPIO LED base on 4-bit hexadecimal value written
proc hps_led_on {pattern} {
  global mj_secu
  global hps_led_base
  set wr_en [secu_rd [ expr $hps_led_base + 4] ]
  secu_wr [ expr $hps_led_base + 4] 0x0000f000
  secu_wr [ expr $hps_led_base ] [ expr $pattern<<12 ]
}

# procedure that light off the 4 HPS GPIO LEDs. Value '1' at GPIO will turn off the lighting
proc hps_led_off {} {
  global mj_secu
  global hps_led_base
  secu_wr [ expr $hps_led_base ] 0x0000f000
}

# procedure set HPS LED to run in single light-on pattern, with an interval set by user
proc hps_led_run {interval occurance} {
  global mj_secu
  global hps_led_base
  secu_wr [ expr $hps_led_base + 4] 0x0000f000
  for {set y 0} {$y<$occurance} {incr y} {
    set value 1    
    for {set x 0} {$x<4} {incr x} {
      set dvalue [expr ~$value]
      secu_wr $hps_led_base [expr $dvalue<<12 ]
      puts " value of LED GPIO register~: 0b [int2bits $value]"
      set value [expr $value <<1 ]
      after $interval
    }
  }
}

# procedure to setup mSGDMA descriptor
proc dma_descp_send {which_dma_base rdaddr wraddr size_in_byte} {
  global mj_dmad
  global tse0_tx_descp_base
  global tse0_rx_descp_base
  global tse1_tx_descp_base
  global tse1_rx_descp_base
  if {$which_dma_base == "tse0_tx"} {
    set dma_base $tse0_tx_descp_base
  } elseif {$which_dma_base == "tse0_rx"} {
    set dma_base $tse0_rx_descp_base
  } elseif {$which_dma_base == "tse1_tx"} {
    set dma_base $tse1_tx_descp_base
  } elseif {$which_dma_base == "tse1_rx"} {
    set dma_base $tse1_rx_descp_base
  }  
  dmad_wr [expr $dma_base ] $rdaddr 1
  dmad_wr [expr $dma_base + 4] $wraddr 1
  dmad_wr [expr $dma_base + 8] $size_in_byte 1
  dmad_wr [expr $dma_base + 28] 0x4300 1
  master_write_8 $mj_dmad [ expr $dma_base + 31 ] 0x80
}

# procedure to setup mSGDMA control
proc dma_enable_irq {which_dma_base} {
  global mj_nsec
  global tse0_tx_csr_base
  global tse0_rx_csr_base
  global tse1_tx_csr_base
  global tse1_rx_csr_base
  if {$which_dma_base == "tse0_tx"} {
    set dma_base $tse0_tx_csr_base
  } elseif {$which_dma_base == "tse0_rx"} {
    set dma_base $tse0_rx_csr_base
  } elseif {$which_dma_base == "tse1_tx"} {
    set dma_base $tse1_tx_csr_base
  } elseif {$which_dma_base == "tse1_rx"} {
    set dma_base $tse1_rx_csr_base
  }  
  set rdata [nsec_rd [expr $dma_base + 4]]
  set wdata [expr $rdata | 0x10]
  nsec_wr [expr $dma_base + 4] $wdata 1
}

proc dma_status {which_dma_base {clearing 0}} {
  global mj_nsec
  global tse0_tx_csr_base
  global tse0_rx_csr_base
  global tse1_tx_csr_base
  global tse1_rx_csr_base
  if {$which_dma_base == "tse0_tx"} {
    set dma_base $tse0_tx_csr_base
  } elseif {$which_dma_base == "tse0_rx"} {
    set dma_base $tse0_rx_csr_base
  } elseif {$which_dma_base == "tse1_tx"} {
    set dma_base $tse1_tx_csr_base
  } elseif {$which_dma_base == "tse1_rx"} {
    set dma_base $tse1_rx_csr_base
  }  
  set rdata [nsec_rd $dma_base]
  if {$clearing == 1} {
    nsec_wr $dma_base $rdata 1
  }
}

# prodecure to enable TSE loopback datapath
proc tse_set_loopback {which_tse_base} {
  global mj_nsec
  global tse0_ctrl_base
  global tse1_ctrl_base
  set sw_reset_set 0x2000
  set sw_reset_clr 0xffffdfff
  set loop_en_set 0x8000
  set tx_en_set 0x1
  set rx_en_set 0x2
  if {$which_tse_base == "tse0"} {
    set tse_base $tse0_ctrl_base
  } elseif {$which_tse_base == "tse1"} {
    set tse_base $tse1_ctrl_base
  } 
  # read command_config register
  set rdata [nsec_rd [expr $tse_base + 8]]
  set wdata [expr $rdata | $sw_reset_set]
  nsec_wr [expr $tse_base + 8] $wdata 1
  
  set rdata [nsec_rd [expr $tse_base + 8]]
  set wdata [expr $rdata | $loop_en_set]
  nsec_wr [expr $tse_base + 8] $wdata 1
  
  set rdata [nsec_rd [expr $tse_base + 8]]
  set wdata [expr $rdata & $sw_reset_clr]
  nsec_wr [expr $tse_base + 8] $wdata 1
  
  set rdata [nsec_rd [expr $tse_base + 8]]
  set wdata [expr $rdata | $tx_en_set | $rx_en_set]
  nsec_wr [expr $tse_base + 8] $wdata 1
}

proc tse_clear_loopback {which_tse_base} {
  global mj_nsec
  global tse0_ctrl_base
  global tse1_ctrl_base
  set sw_reset_set 0x2000
  set sw_reset_clr 0xffffdfff
  set loop_en_clr 0xffff7fff
  if {$which_tse_base == "tse0"} {
    set tse_base $tse0_ctrl_base
  } elseif {$which_tse_base == "tse1"} {
    set tse_base $tse1_ctrl_base
  } 
  # read command_config register
  set rdata [nsec_rd [expr $tse_base + 8]]
  set wdata [expr $rdata | $sw_reset_set]
  nsec_wr [expr $tse_base + 8] $wdata 1
  
  set rdata [nsec_rd [expr $tse_base + 8]]
  set wdata [expr $rdata & $loop_en_clr]
  nsec_wr [expr $tse_base + 8] $wdata 1
  
  set rdata [nsec_rd [expr $tse_base + 8]]
  set wdata [expr $rdata & $sw_reset_clr]
  nsec_wr [expr $tse_base + 8] $wdata 1
}

# procedure to check current transfer status
proc tse_check_status {which_tse_base} {
  global mj_nsec
  global tse0_ctrl_base
  global tse1_ctrl_base
  if {$which_tse_base == "tse0"} {
    set tse_base $tse0_ctrl_base
  } elseif {$which_tse_base == "tse1"} {
    set tse_base $tse1_ctrl_base
  } 
  set rdata [nsec_rd [expr $tse_base + 0x68]]
  puts "Transmitted OK frame: $rdata"
  set rdata [nsec_rd [expr $tse_base + 0x6c]]
  puts "Received OK frame:    $rdata"
  set rdata [nsec_rd [expr $tse_base + 0x8c]]
  puts "Transmitted frame with Error: $rdata"
  set rdata [nsec_rd [expr $tse_base + 0x88]]
  puts "Received frame with Error:    $rdata"
}

# procedure to detect phy address
proc tse_detect_phy {which_tse_base} {
  global mj_nsec
  global tse0_ctrl_base
  global tse1_ctrl_base
  if {$which_tse_base == "tse0"} {
    set tse_base $tse0_ctrl_base
  } elseif {$which_tse_base == "tse1"} {
    set tse_base $tse1_ctrl_base
  } 
  set mt_cnt 0
  set mm_cnt 0
  # soft reset PHY. MDIO Space 0 (0x80 dword addr, 0x200) or MDIO Space 1 (0xa0 dword addr, 0x280)
  set mdio_base 0x200
  for {set i 0} {$i<32} {incr i} {
    # write address to mdio_addr0 (0x3c) or mdio_addr1 (0x40)
    nsec_wr [expr $tse_base + 0x3c] $i 1
        
    nsec_wr [expr $tse_base + $mdio_base] 0x8000 1
    set rdata_r2 [nsec_rd [expr $tse_base + $mdio_base + 0x8]]
    puts "address 0x[format %x [expr $mdio_base + 0x8]] value: $rdata_r2"
    set rdata_r3 [nsec_rd [expr $tse_base + $mdio_base + 0xc]]
    puts "address 0x[format %x [expr $mdio_base + 0xc]] value: $rdata_r3"
    if {$rdata_r2 == $rdata_r3} {
      set mt_cnt [ expr $mt_cnt+1 ]
    } else {
      set mm_cnt [ expr $mm_cnt+1 ]
    }
  }
  puts "has different [format %x [expr [expr $mdio_base + 0x8]/4]] vs. [format %x [expr [expr $mdio_base + 0xc]/4]] value: $mm_cnt"  
  puts "identical [format %x [expr [expr $mdio_base + 0x8]/4]] & [format %x [expr [expr $mdio_base + 0xc]/4]] count: $mt_cnt"
}

# procedure to configure TSE MAC
proc setup_tse_mac {which_tse_base} {
  global mj_nsec
  global tse0_ctrl_base
  global tse1_ctrl_base
  if {$which_tse_base == "tse0"} {
    set tse_base $tse0_ctrl_base
  } elseif {$which_tse_base == "tse1"} {
    set tse_base $tse1_ctrl_base
  } 
  set MAC_0 				0x22334450
  set MAC_1 				0x0000EE11
  set FRM_LENGTH  		1518
  set PAUSE_QUANT  		8192
  set RX_SECTION_EMPTY 	8172
  set RX_SECTION_FULL  	16
  set TX_SECTION_EMPTY 	8172
  set TX_SECTION_FULL  	16
  set RX_ALMOST_EMPTY 	8
  set RX_ALMOST_FULL  	8
  set TX_ALMOST_EMPTY  	8
  set TX_ALMOST_FULL    3
  set MDIO_ADDR0  		0
  set MDIO_ADDR1  		0
  set jtag_master $mj_nsec
  # COMMAND_CONFIG
  set ENA_TX 				1
  set ENA_RX 				1
  set XON_GEN 			0
  set ETH_SPEED 			1;	# 10/100Mbps = 0 & 1000Mbps = 1
  set PROMIS_EN			1;
  set PAD_EN 				1;
  set CRC_FWD 			0;
  set PAUSE_FWD 			0;
  set PAUSE_IGNORE		0;
  set TX_ADDR_INS 		0;
  set HD_ENA 				0;
  set EXCESS_COL 		0;
  set LATE_COL			0;
  set SW_RESET 			0;
  set MHASH_SEL			0;
  set LOOP_ENA			1;	#0;
  set TX_ADDR_SEL 		0x0;
  set MAGIC_ENA			0;
  set SLEEP	 			0;
  set WAKEUP 				0;
  set XOFF_GEN 			0;
  set CTRL_FRM_ENA 		0;
  set NO_LGTH_CHECK		0;
  set ENA_10 				0; # 100Mbps = 0 & 10Mbps = 1
  set RX_ERR_DISC 		0;
  set DISABLE_RD_TIMEOUT	0;
  set CNT_RESET			0;

  set TX_IPG_LENGTH 		12;
  set TX_OMIT_CRC 		0;
  set TX_SHIFT16 			0; 
  set RX_SHIFT16 			0;
  source tse_mac_config.tcl
  
  # PHY MISC
set PHY_ENABLE 			1;		# Enable PHY Port 0

# PHY Configuration
set PHY_ETH_SPEED 		100;	# 10Mbps or 100Mbps or 1000Mbps
set PHY_ENABLE_AN 		1;		# Enable PHY Auto-Negotiation
set PHY_COPPER_DUPLEX	1;		# FD = 1 and HD = 0
  source tse_marvel_phy.tcl
  
}

######################### information #################################
# add your new procedure here to access newly added component in system
